bin_path     = "{prefix}/bin"
license_path = "{prefix}/share/doc/tortoisehg/COPYING.txt"
locale_path  = "{python.prefix}/share/locale"
icon_path    = "{python.prefix}/share/pixmaps/tortoisehg/icons"
nofork       = True
