# -*- coding: utf-8; mode: tcl; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:ft=tcl:et:sw=4:ts=4:sts=4

pre-fetch {
    ui_warn "Port maintainers: the compiler_blacklist_versions PortGroup is obsolete and should be removed from Portfiles."
}

# This PortGroup is scheduled for deletion after: 2026-08-12
